<?php

declare(strict_types=1);

namespace Drupal\graphql_export\Commands;

use Consolidation\AnnotatedCommand\CommandData;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\graphql\Entity\Server;
use Drupal\graphql_export\GraphqlExportService;
use Drush\Commands\DrushCommands;

/**
 * Adds the post-command hook to the config:export command.
 */
class GraphqlExportCommands extends DrushCommands {

  use StringTranslationTrait;

  /**
   * Construct a new export controller.
   *
   * @param \Drupal\graphql_export\GraphqlExportService $exportService
   *   The export service.
   */
  public function __construct(
    protected GraphqlExportService $exportService
  ) {}

  /**
   * Export the GraphQL Schemas.
   *
   * @hook post-command config:export
   */
  public function postCommand($result, CommandData $commandData): void {
    $settings = Settings::get('graphql_export', []);
    if (empty($settings)) {
      return;
    }

    $servers = Server::loadMultiple();

    foreach ($servers as $server) {
      if (!isset($settings[$server->id()])) {
        continue;
      }

      $config = $settings[$server->id()];

      if (isset($config['graphqls'])) {
        try {
          $data = $this->exportService->printSchema($server);
          $this->writeFile($config['graphqls'], $data);
        }
        catch (\Exception $e) {
          $this->logger()->error($this->t('Error exporting GraphQL schema: @message', ['@message' => $e->getMessage()]));
        }
      }

      if (isset($config['json'])) {
        try {
          $data = $this->exportService->printJson($server);
          $this->writeFile($config['json'], $data);
        }
        catch (\Exception $e) {
          $this->logger()->error($this->t('Error exporting GraphQL json: @message', ['@message' => $e->getMessage()]));
        }
      }
    }
  }

  /**
   * Write the file.
   *
   * @param string $path
   *   The path.
   * @param string $data
   *   The data.
   */
  private function writeFile(string $path, string $data): void {
    $this->logger()->notice($this->t('Writing GraphQL schema to @path', ['@path' => $path]));
    file_put_contents($path, $data);
  }

}
